from typing import Dict, Optional, Tuple

import re
import multipart

class Part:
    headers = []
    data : bytes

    def __repr__(self):
        return f"Part(headers={repr(self.headers)}, data={self.data})"

class Header:
    name : bytes
    value : bytes

    def __repr__(self):
        return f"{self.name}:{self.value}"

def get_body_from_form_data(
    body: bytes, boundary: str
) -> Tuple[Optional[bytes], Dict[str, str]]:
    print("YO BYTES", body)

    current_part = Part()
    parts = []

    def on_part_begin():
        current_part = Part()
        print("YO PART BEGAN")
        return

    def on_part_data(b,s,e):
        current_part.data = b[s:e]
        return

    def on_part_end():
        parts.append(current_part)
        return

    def on_header_begin():

        return

    def on_header_end():
        return

    def on_header_field(b,s,e):
        h = Header()
        current_part.headers.append(h)
        current_part.headers[-1].name = b[s:e]
        return

    def on_header_value(b,s,e):
        current_part.headers[-1].value = b[s:e]
        return

    def on_headers_finished():
        return

    def on_end():
        return

    parser = multipart.MultipartParser(
        boundary=boundary,
        callbacks={
            "on_part_begin": on_part_begin,
            "on_part_data": on_part_data,
            "on_part_end": on_part_end,
            "on_header_field": on_header_field,
            "on_header_value": on_header_value,
            "on_header_end": on_header_end,
            "on_headers_finished": on_headers_finished,
            "on_end": on_end,
        }
    )
    parser.write(body)
    parser.finalize()

    data = None
    headers: Dict[str, str] = {}

    for part in parts:
        for header in part.headers:
            print("YO CHECK HEADER", header.value)
            if b'name="upload_file"' in header.value:
                headers["key"] = "upload_file"
                data = part.data
            else:
                if b'name="file"' in header.value:
                    data = part.data
                else:
                    match = re.search(r'name="([^"]*)"', header.value.decode("utf-8"))
                    if match:
                        name = match.group(1)
                        headers[name] = part.data.decode("utf-8")
    print("YO HEADERS", headers)
    print("YO DATA", data)
    return data, headers
